<?php

final class PhabricatorFlagEditController extends PhabricatorFlagController {

  private $phid;

  public function willProcessRequest(array $data) {
    $this->phid = $data['phid'];
  }

  public function processRequest() {
    $request = $this->getRequest();
    $user = $request->getUser();

    $phid = $this->phid;
    $handle = PhabricatorObjectHandleData::loadOneHandle($phid, $user);

    if (!$handle->isComplete()) {
      return new Aphront404Response();
    }

    $flag = PhabricatorFlagQuery::loadUserFlag($user, $phid);

    if (!$flag) {
      $flag = new PhabricatorFlag();
      $flag->setOwnerPHID($user->getPHID());
      $flag->setType($handle->getType());
      $flag->setObjectPHID($handle->getPHID());
      $flag->setReasonPHID($user->getPHID());
    }

    if ($request->isDialogFormPost()) {
      $flag->setColor($request->getInt('color'));
      $flag->setNote($request->getStr('note'));
      $flag->save();

      return id(new AphrontReloadResponse())->setURI('/flag/');
    }

    $type_name = $handle->getTypeName();

    $dialog = new AphrontDialogView();
    $dialog->setUser($user);

    $dialog->setTitle("Flag {$type_name}");

    require_celerity_resource('phabricator-flag-css');

    $form = new AphrontFormLayoutView();

    $is_new = !$flag->getID();

    if ($is_new) {
      $form
        ->appendChild(
          "<p>You can flag this {$type_name} if you want to remember to look ".
          "at it later.</p><br />");
    }

    $radio = new AphrontFormRadioButtonControl();
    foreach (PhabricatorFlagColor::getColorNameMap() as $color => $text) {
      $class = 'phabricator-flag-radio phabricator-flag-color-'.$color;
      $radio->addButton($color, $text, '', $class);
    }

    $form
      ->appendChild(
        $radio
          ->setName('color')
          ->setLabel('Flag Color')
          ->setValue($flag->getColor()))
      ->appendChild(
        id(new AphrontFormTextAreaControl())
          ->setHeight(AphrontFormTextAreaControl::HEIGHT_VERY_SHORT)
          ->setName('note')
          ->setLabel('Note')
          ->setValue($flag->getNote()));

    $dialog->appendChild($form);

    $dialog->addCancelButton($handle->getURI());
    $dialog->addSubmitButton(
      $is_new ? "Flag {$type_name}" : 'Save');

    return id(new AphrontDialogResponse())->setDialog($dialog);
  }

}
